{% extends "layout.html" %}
{% block body %}
<h2>Page Not Found</h2>
<p>
  The requested page does not exist on this server.  If you expect something
  here (for example a server) it probably went away after the last update.
</p>
<p>
  go back to <a href="{{ url_for('serverlist') }}">the server list</a>.
</p>
{% endblock %}
