{% extends "layout.html" %}
{% block body %}
  <div class="page">
    <h2>About Plnt</h2>
    <p>
      Plnt is a small example application written using the
      <a href="http://werkzeug.pocoo.org/">Werkzeug</a> WSGI toolkit,
      the <a href="http://jinja.pocoo.org/">Jinja</a> template language,
      the <a href="https://www.sqlalchemy.org/">SQLAlchemy</a> database abstraction
      layer and ORM and last but not least the awesome
      <a href="http://feedparser.org/">feedparser</a> library.
    </p>
    <p>
      It's one of the example applications developed to show some of the
      features werkzeug provides and could be the base of a real planet
      software.
    </p>
  </div>
{% endblock %}
